% vingenc.m - PFN Type C General
% J. Clementson 2013-04-22

clear

prompt = {'LOAD CHARACTERISTICS'; ...
    ' R_{load} [Ohm] '; ...
    'L_{load} [Henry]'; ...
    '----------||----------||----------||----------||----------||----------||----------||----------||----------||----------'; ...
    'NETWORK CHARACTERISTICS';...
    'Type of Network'; ...
    'C_i [Farad]   '; ...
    'L_i [Henry]   '; ...
    ' Lossy Network'; ...
    'R_i [Ohm]     '; ...
    ' U_0 [Volt]   '; ...
    'R_{term} [Ohm]  '; ...
    ' N [cells]   '; ...
    'L_{term} [Henry]'; ...
     '----------||----------||----------||----------||----------||----------||----------||----------||----------||----------'; ...
    'DATA OUTPUT';...
    ' T_{int} [s]    '; ...
    ' Plot Load Current'; ...
    ' Plot Capacitor Voltages'; ...
    ' Plot Capacitor Currents'; ...
    ' Plot Inductor Currents'; ...
    ' Save Load Current Data'; ...
    ' Directory   '; ...
    ' File Name [.txt]     '};

name = 'PFN TYPE C GENERAL';

formats = struct('type', {}, 'style', {}, 'items', {}, ...
  'format', {}, 'limits', {}, 'size', {});

% Load Characteristics
formats(3, 1).type = 'text';

% Load Resistance
formats(5, 1).type   = 'edit';
formats(5, 1).format = 'float';
formats(5, 1).size = 60;

% Load Inductance
formats(5, 2).type = 'edit';
formats(5, 2).format = 'float';
formats(5, 2).size = 60;

% Network Characteristics
formats(7, 1).type = 'text';
formats(7, 2).type = 'none';
formats(7, 2).limits = [0, 10];
formats(7, 3).type = 'none';
formats(7, 3).limits = [0, 10];
formats(8, 1).type = 'text';
formats(8, 2).type = 'none';
formats(8, 2).limits = [0, 10];
formats(8, 3).type = 'none';
formats(8, 3).limits = [0, 10];

% Network Option
formats(10, 1).type = 'list';
formats(10, 1).style = 'radiobutton';
formats(10, 1).items = {' Equicell                      '; ' Unicell'};
formats(11, 1).type = 'none';
formats(11, 1).limits = [10, 0];

% Cell Capacitors
formats(10, 2).type = 'edit';
formats(10, 2).format = 'text';
formats(10, 2).size = 420;
formats(10, 3).type = 'none';
formats(10, 3).limits = [0, 10];

% Cell Inductors
formats(11, 2).type = 'edit';
formats(11, 2).format = 'text';
formats(11, 2).size = 420;
formats(11, 3).type = 'none';
formats(11, 3).limits = [0, 10];

% Lossy Network
formats(12, 1).type = 'check';
formats(12, 1).style = 'checkbox';

% Cell Resistors
formats(12, 2).type = 'edit';
formats(12, 2).format = 'text';
formats(12, 2).size = 420;
formats(12, 3).type = 'none';
formats(12, 3).limits = [0, 10];

% Voltage Source
formats(13, 1).type = 'edit';
formats(13, 1).format = 'float';
formats(13, 1).size = 60;

% Termination Resistor
formats(13, 2).type = 'edit';
formats(13, 2).format = 'float';
formats(13, 2).size = 60;

% Number of Cells
formats(15, 1).type = 'edit';
formats(15, 1).format = 'float';
formats(15, 1).size = 60;

% Termination Inductor
formats(15, 2).type = 'edit';
formats(15, 2).format = 'float';
formats(15, 2).size = 60;

% Data Output
formats(17, 1).type = 'text';
formats(17, 2).type = 'none';
formats(17, 2).limits = [0, 10];
formats(17, 3).type = 'none';
formats(17, 3).limits = [0, 10];
formats(18, 1).type = 'text';
formats(18, 2).type = 'none';
formats(18, 2).limits = [0, 10];
formats(18, 3).type = 'none';
formats(18, 3).limits = [0, 10];

% Time Interval
formats(20, 1).type = 'edit';
formats(20, 1).format = 'float';
formats(20, 1).size = 60;

% Plot options
formats(22, 1).type = 'check';
formats(22, 1).style = 'checkbox';
formats(24, 1).type = 'check';
formats(24, 1).style = 'checkbox';
formats(24, 2).type = 'check';
formats(24, 2).style = 'checkbox';
formats(24, 3).type = 'check';
formats(24, 3).style = 'checkbox';

% Data to file
formats(26, 1).type = 'check';
formats(26, 1).style = 'checkbox';

% Directory
formats(26, 2).type = 'edit';
formats(26, 2).format = 'dir';
formats(26, 2).size = 120;

% File name
formats(26, 3).type = 'edit';
formats(26, 3).format = 'text';
formats(26, 3).size = 120;

defaultanswer = {'', ...        % text      1
    1, ...                      % R_L       2
    0, ...                      % L_L       3
    '', ...                     % frame     4
    '', ...                     % text      5
    1, ...                      % equi/uni  6
    '1e-6', ...                 % C_i       7
    '1e-6', ...                 % L_i       8
    0, ...                      % Lossy Net 9
    '0', ...                    % R_i       10
    1000, ...                   % U_0       11
    0, ...                      & R_term    12
    1, ...                      % N         13
    0, ...                      & L_term    14
    '', ...                     % frame     15
    '', ...                     % text      16
    50e-6, ...                  % T_int     17
    1, ...                      % plot I_L  18
    0, ...                      % plot U_cap19
    0, ...                      % plot I_cap20
    0, ...                      % plot I_ind21
    0, ...                      % save data 22
    '', ...                     & directory 23
    ''};                        % file      24

options.Resize = 'off';
options.WindowStyle = 'normal';
options.Interpreter = 'tex';
options.CancelButton = 'on';
options.ApplyButton = 'off';
options.AlignControls = 'on';

[answer, canceled] = inputsdlg(prompt, name, formats, defaultanswer, options);

R_L = answer{2};
L_L = answer{3};
U_0 = answer{11};
N = answer{13};
u0 = U_0*ones(1, 2*N);
u0(1, 2:2:2*N) = 0;
R_term = answer{12};
L_term = answer{14};
C = str2num(answer{7});
L = str2num(answer{8});
R = str2num(answer{10});
if answer{9} == 0
    R = zeros(1, N);
end
if answer{6} == 1
    C = C(1)*ones(1, N);
    L = L(1)*ones(1, N);
    R = R(1)*ones(1, N);
else
    if answer{9} == 0
        if length(C) ~= N || length(L) ~= N
            display 'The number of capacitors and inductors must equal the number of cells.'
            return
        end
    else
        if length(C) ~= N || length(L) ~= N || length(R) ~= N
            display 'The number of capacitors, inductors, and resistors must equal the number of cells.'
            return
        end
    end
end
T_int = 0: answer{17}/999: answer{17};

if canceled == true
    return
end

[t, u] = ode45(@pfnc, T_int, u0, [], R_L, L_L, L, C, R, N, R_term, L_term);
i = zeros(length(t), N);
for k = 1: N
    i(:, k) = -C(k)*u(:, 2*k);
end
I_L = sum(i, 2);

L_inv = 0;
C_tot = 0;
R_inv = 0;
for k = 1: N
    L_inv = L_inv + 1/L(k);
    C_tot = C_tot + C(k);
    R_inv = R_inv + 1/R(k);
end
L_tot = 1/L_inv;
R_tot = 1/R_inv;
if answer{9} == 0
    Z_0 = sqrt(L_tot/C_tot);
end


%%%%%%%%%% Plot Load Current %%%%%%%%%%

if answer{18} == 1
    figure('Color', 1/255*[50 200 250]);
    hold on;
    plot(t, I_L, 'LineWidth', 2)
    hold on;
    if answer{6} == 1
        network_text = ['  Equicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C(1)), ' F' 10 ...
        '  L_{i} = ', num2str(L(1)), ' H' 10 ...
        '  R_{i} = ', num2str(R(1)), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = 1;
    else
        network_text = ['  Unicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C), ' F' 10 ...
        '  L_{i} = ', num2str(L), ' H' 10 ...
        '  R_{i} = ', num2str(R), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = N;
    end
    axPos = get(gca,'Position');
    x_start_end = xlim;
    y_start_end = ylim;
    x_norm = axPos(1) + (T_int(1000)*1.1-step*T_int(1000)/15 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
    y_norm = axPos(2) + (0.9*y_start_end(2) - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
    x_rel = x_norm*(x_start_end(2) - x_start_end(1)) + x_start_end(1);
    y_rel = y_norm*(y_start_end(2) - y_start_end(1)) + y_start_end(1);
    text(x_rel, y_rel, network_text);
    if answer{9} == 0
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm' 10 ...
        '  Z_0 = ', num2str(Z_0), ' Ohm'];
    else
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm'];
    end
    text(x_rel, y_rel*0.6, network_text2);
    xlabel('t (s)')
    ylabel('I_{load} (A)')
    title(['PFN Type C with U_0 = ', num2str(U_0), ' V', ...
    '' 10 ...
    'Discharging over a load with R_{load} = ', num2str(R_L), ' Ohm and L_{load} = ', num2str(L_L), ' Henry'],  ...
    'FontWeight','bold', 'HorizontalAlignment', 'center')
    plot(t, 0)
end


%%%%%%%%%% Plot Capacitor Voltages %%%%%%%%%%

if answer{19} == 1
    figure('Color', 1/255*[50 200 250]);
    plot(t, u(:, 1:2:2*N-1), 'LineWidth', 2)
    hold on;
    if answer{6} == 1
        network_text = ['  Equicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C(1)), ' F' 10 ...
        '  L_{i} = ', num2str(L(1)), ' H' 10 ...
        '  R_{i} = ', num2str(R(1)), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = 1;
    else
        network_text = ['  Unicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C), ' F' 10 ...
        '  L_{i} = ', num2str(L), ' H' 10 ...
        '  R_{i} = ', num2str(R), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = N;
    end
    axPos = get(gca,'Position');
    x_start_end = xlim;
    y_start_end = ylim;
    x_norm = axPos(1) + (T_int(1000)*1.1-step*T_int(1000)/15 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
    y_norm = axPos(2) + (0.9*y_start_end(2) - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
    x_rel = x_norm*(x_start_end(2) - x_start_end(1)) + x_start_end(1);
    y_rel = y_norm*(y_start_end(2) - y_start_end(1)) + y_start_end(1);
    text(x_rel, y_rel, network_text);
    if answer{9} == 0
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm' 10 ...
        '  Z_0 = ', num2str(Z_0), ' Ohm'];
    else
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm'];
    end
    text(x_rel, y_rel*0.6, network_text2);
    cu_text = cell(1, N);
    for k = 1: N
        cu_text{k} = strcat('C_{', num2str(k), '}');
    end
    legend(cu_text, 'location', 'East')
    xlabel('t (s)')
    ylabel('Capacitor voltage (V)')
    title(['PFN Type C with U_0 = ', num2str(U_0), ' V', ...
    '' 10 ...
    'Discharging over a load with R_{load} = ', num2str(R_L), ' Ohm and L_{load} = ', num2str(L_L), ' Henry'],  ...
    'FontWeight','bold', 'HorizontalAlignment', 'center')
    plot(t, 0)
end


%%%%%%%%%% Plot Capacitor Currents %%%%%%%%%%

if answer{20} == 1
    figure('Color', 1/255*[50 200 250]);
    plot(t, i, 'LineWidth', 2)
    hold on;
    if answer{6} == 1
        network_text = ['  Equicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C(1)), ' F' 10 ...
        '  L_{i} = ', num2str(L(1)), ' H' 10 ...
        '  R_{i} = ', num2str(R(1)), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = 1;
    else
        network_text = ['  Unicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C), ' F' 10 ...
        '  L_{i} = ', num2str(L), ' H' 10 ...
        '  R_{i} = ', num2str(R), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = N;
    end
    axPos = get(gca,'Position');
    x_start_end = xlim;
    y_start_end = ylim;
    x_norm = axPos(1) + (T_int(1000)*1.1-step*T_int(1000)/15 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
    y_norm = axPos(2) + (0.9*y_start_end(2) - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
    x_rel = x_norm*(x_start_end(2) - x_start_end(1)) + x_start_end(1);
    y_rel = y_norm*(y_start_end(2) - y_start_end(1)) + y_start_end(1);
    text(x_rel, y_rel, network_text);
    if answer{9} == 0
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm' 10 ...
        '  Z_0 = ', num2str(Z_0), ' Ohm'];
    else
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm'];
    end
    text(x_rel, y_rel*0.6, network_text2);
    ci_text = cell(1, N);
    for k = 1: N
        ci_text{k} = strcat('C_{', num2str(k), '}');
    end
    legend(ci_text, 'location', 'East')
    xlabel('t (s)')
    ylabel('Capacitor current (A)')
    title(['PFN Type C with U_0 = ', num2str(U_0), ' V', ...
    '' 10 ...
    'Discharging over a load with R_{load} = ', num2str(R_L), ' Ohm and L_{load} = ', num2str(L_L), ' Henry'],  ...
    'FontWeight','bold', 'HorizontalAlignment', 'center')
    plot(t, 0)
end


%%%%%%%%%% Plot Inductor Currents %%%%%%%%%%

if answer{21} == 1
    figure('Color', 1/255*[50 200 250]);
    plot(t, i, 'LineWidth', 2)
    hold on;
    if answer{6} == 1
        network_text = ['  Equicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C(1)), ' F' 10 ...
        '  L_{i} = ', num2str(L(1)), ' H' 10 ...
        '  R_{i} = ', num2str(R(1)), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = 1;
    else
        network_text = ['  Unicell Network' 10 ...
        '  ----------------' 10 ...
        '  N = ', num2str(N), ' cells' 10 ...
        '  C_{i} = ', num2str(C), ' F' 10 ...
        '  L_{i} = ', num2str(L), ' H' 10 ...
        '  R_{i} = ', num2str(R), ' Ohm' 10 ...
        '  R_{term} = ', num2str(R_term), ' Ohm' 10 ...
        '  L_{term} = ', num2str(L_term), ' H'];
        step = N;
    end
    axPos = get(gca,'Position');
    x_start_end = xlim;
    y_start_end = ylim;
    x_norm = axPos(1) + (T_int(1000)*1.1-step*T_int(1000)/15 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
    y_norm = axPos(2) + (0.9*y_start_end(2) - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
    x_rel = x_norm*(x_start_end(2) - x_start_end(1)) + x_start_end(1);
    y_rel = y_norm*(y_start_end(2) - y_start_end(1)) + y_start_end(1);
    text(x_rel, y_rel, network_text);
    if answer{9} == 0
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm' 10 ...
        '  Z_0 = ', num2str(Z_0), ' Ohm'];
    else
        network_text2 = ['  C = ', num2str(C_tot), ' F' 10 ...
        '  L = ', num2str(L_tot), ' H' 10 ...
        '  R = ', num2str(R_tot), ' Ohm'];
    end
    text(x_rel, y_rel*0.6, network_text2);
    li_text = cell(1, N);
    for k = 1: N
        li_text{k} = strcat('L_{', num2str(k), '}');
    end
    legend(li_text, 'location', 'East')
    xlabel('t (s)')
    ylabel('Inductor current (A)')
    title(['PFN Type C with U_0 = ', num2str(U_0), ' V', ...
    '' 10 ...
    'Discharging over a load with R_{load} = ', num2str(R_L), ' Ohm and L_{load} = ', num2str(L_L), ' Henry'],  ...
    'FontWeight','bold', 'HorizontalAlignment', 'center')
    plot(t, 0)
end


%%%%%%%%%% Save Load Current Data %%%%%%%%%%

if answer{22} == 1
    dir = answer{23};
    file = answer{24};
    if strcmp(dir, '') || strcmp(file, '')
        disp('Missing directory or file.')
        return
    end
    path = [dir, '/', file, '.txt'];
    tI_L = [t I_L];
    save(path, 'tI_L','-ascii');
end