% vinoptb.m - PFN Type B Optimization
% J. Clementson 2013-04-22

clear

prompt = {'LOAD CHARACTERISTICS'; ...
    ' R_{load} [Ohm]   '; ...
    'L_{load} [Henry]'; ...
    '----------||----------||----------||----------||----------||----------||----------||----------||----------||----------'; ...
    'NETWORK CHARACTERISTICS';...
    'Type of Network'; ...
    'T [s]'; ...
    'C_i [Farad]   '; ...
    'I_{ripple} [%]'; ...
    'L_{imax} [Henry]'; ...
    ' U_0 [Volt]   '; ...
    'R_{term} [Ohm]'; ...
    ' N_{max} [cells]  '; ...
    'L_{term} [Ohm]'; ...
    '----------||----------||----------||----------||----------||----------||----------||----------||----------||----------'; ...
    'DATA OUTPUT';...
    ' T_{int} [s]      '; ...
    ' Plot Load Current'; ...
    ' Save Load Current Data'; ...
    ' Directory   '; ...
    ' File Name [.txt]     '};

name = 'PFN TYPE B OPTIMIZATION';

formats = struct('type', {}, 'style', {}, 'items', {}, ...
  'format', {}, 'limits', {}, 'size', {});

% Load Characteristics
formats(3, 1).type = 'text';

% Load Resistance
formats(5, 1).type   = 'edit';
formats(5, 1).format = 'float';
formats(5, 1).size = 60;

% Load Inductance
formats(5, 2).type = 'edit';
formats(5, 2).format = 'float';
formats(5, 2).size = 60;

% Network Characteristics
formats(7, 1).type = 'text';
formats(7, 2).type = 'none';
formats(7, 2).limits = [0, 10];
formats(7, 3).type = 'none';
formats(7, 3).limits = [0, 10];
formats(8, 1).type = 'text';
formats(8, 2).type = 'none';
formats(8, 2).limits = [0, 10];
formats(8, 3).type = 'none';
formats(8, 3).limits = [0, 10];

% Network Option
formats(10, 1).type = 'list';
formats(10, 1).style = 'radiobutton';
formats(10, 1).items = {' Rayleigh                      '};
formats(11, 1).type = 'none';
formats(11, 1).limits = [10, 0];
formats(12, 1).type = 'none';
formats(12, 1).limits = [10, 0];

% Pulse Length
formats(10, 2).type = 'edit';
formats(10, 2).format = 'float';
formats(10, 2).size = 60;

% Cell Capacitors
formats(10, 3).type   = 'list';
formats(10, 3).style  = 'listbox';
formats(10, 3).size = 120;
formats(10, 3).limits = [0, 10];
caplist = importdata('caplist.txt');
formats(10, 3).items  = caplist;
formats(11, 3).type = 'none';
formats(11, 3).limits = [10, 0];
formats(12, 3).type = 'none';
formats(12, 3).limits = [10, 0];
formats(13, 3).type = 'none';
formats(13, 3).limits = [10, 0];
formats(14, 3).type = 'none';
formats(14, 3).limits = [10, 0];

% Pulse Ripple
formats(11, 2).type = 'edit';
formats(11, 2).format = 'float';
formats(11, 2).size = 60;

% Max Cell Inductance
formats(12, 2).type = 'edit';
formats(12, 2).format = 'float';
formats(12, 2).size = 60;

% Voltage Source
formats(13, 1).type = 'edit';
formats(13, 1).format = 'float';
formats(13, 1).size = 60;

% Termination Resistor
formats(13, 2).type = 'edit';
formats(13, 2).format = 'float';
formats(13, 2).size = 60;

% Max Number of Cells
formats(14, 1).type = 'edit';
formats(14, 1).format = 'float';
formats(14, 1).size = 60;

% Termination Inductor
formats(14, 2).type = 'edit';
formats(14, 2).format = 'float';
formats(14, 2).size = 60;

% Data Output
formats(16, 1).type = 'text';
formats(16, 2).type = 'none';
formats(16, 2).limits = [0, 10];
formats(16, 3).type = 'none';
formats(16, 3).limits = [0, 10];
formats(17, 1).type = 'text';
formats(17, 2).type = 'none';
formats(17, 2).limits = [0, 10];
formats(17, 3).type = 'none';
formats(17, 3).limits = [0, 10];

% Time Interval
formats(19, 1).type = 'edit';
formats(19, 1).format = 'float';
formats(19, 1).size = 60;

% Plot options
formats(21, 1).type = 'check';
formats(21, 1).style = 'checkbox';

% Data to file
formats(23, 1).type = 'check';
formats(23, 1).style = 'checkbox';

% Directory
formats(23, 2).type = 'edit';
formats(23, 2).format = 'dir';
formats(23, 2).size = 120;

% File name
formats(23, 3).type = 'edit';
formats(23, 3).format = 'text';
formats(23, 3).size = 120;

defaultanswer = {'', ...        % text      1
    1, ...                      % R_L       2
    0, ...                      % L_L       3
    '', ...                     % frame     4
    '', ...                     % text      5
    1, ...                      % Rayleigh  6
    10e-6, ...                  % T         7
    '', ...                     % C_i       8
    10, ...                     % ripple    9
    10e-4, ...                  % L_imax    10
    1000, ...                   % U_0       11
    0, ...                      % R_term    12
    1, ...                      % N_max     13
    0, ...                      & L_term    14
    '', ...                     % frame     15
    '', ...                     % text      16
    50e-6, ...                  % T_int     17
    1, ...                      % plot I_L  18
    0, ...                      % save data 19
    '', ...                     & directory 20
    ''};                        % file      21

options.Resize = 'off';
options.WindowStyle = 'normal';
options.Interpreter = 'tex';
options.CancelButton = 'on';
options.ApplyButton = 'off';
options.AlignControls = 'on';

[answer, canceled] = inputsdlg(prompt, name, formats, defaultanswer, options);

R_L = answer{2};
L_L = answer{3};
T = answer{7};
C_list = caplist(answer{8});
I_rip = answer{9}/100;
U_0 = answer{11};
L_imax = answer{10};
R_term = answer{12};
I_nom = U_0/2/(R_L+R_term);
I_low = I_nom - I_nom*I_rip;
I_high = I_nom + I_nom*I_rip;
N_max = answer{13};
L_term = answer{14};
T_int = [0 answer{17}];
if T_int(2) < T
    disp('Simulation length must be longer than pulse length.')
    return
end

if canceled == true
    return
end


%%%%%%%%%% RAYLEIGH OPTIMIZATION %%%%%%%%%%

if answer{6} == 1
    if answer{18} == 1
        figure('Color', 1/255*[250 250 150]);
        hold on;
    end
    L_list = (R_term + R_L)^2*C_list;
    for k = 1: length(C_list)
        if L_list(k) > L_imax
            L_list(k) = L_imax;
        end
    end
    I_list = cell(1, length(C_list));
    N = zeros(1, length(C_list));
    Z_0 = zeros(1, length(C_list));
    for j = 1: length(C_list)
        N(j) = (T*1.1)/2/sqrt(L_list(j)*C_list(j));
        N(j) = ceil(N(j));
        t_pulse = 0;
        while t_pulse < T && N(j) <= N_max
            u0 = U_0*ones(1, 2*N(j));
            u0(1, 2:2:2*N(j)) = 0;
            L = L_list(j)*ones(1, N(j));
            C = C_list(j)*ones(1, N(j));
            R = zeros(1, N(j));
            [t, u] = ode45(@pfnb, T_int, u0, [], R_L, L_L, L, C, R, N(j), R_term, L_term);
            i = -C_list(j)*u(:, 2:2:2*N(j));
            I_L = sum(i, 2);
            I_list{j} = [t, I_L];
            m = 0;
            t1 = 0;
            t2 = 0;
            for n = 1: length(I_L)
                if I_low <= I_L(n) && I_L(n) <= I_high && t_pulse < T
                    m = m + 1;
                    if m == 1
                        t1 = t(n);
                    else
                        t2 = t(n);
                        t_pulse = t2 - t1;
                    end
                else
                    m = 0;
                end
            end
            if t_pulse < T
                N(j) = N(j) + 1;
            end
        end
        Z_0(j) = sqrt(L_list(j)/C_list(j));
        
        %%%%%%%%%% Plot Load Currents %%%%%%%%%%
        
        if answer{18} == 1
            subplot(length(C_list), 1, j)
            if j == 1
                title(['PFN Type B with U_0 = ', num2str(U_0), ' V, N_{max} = ', num2str(N_max), ' cells, T_{pulse} > ', num2str(T), ' s, I_{ripple} < ', num2str(I_rip*100), ' %, L_{imax} = ', num2str(L_imax), ' H, R_{term} = ', num2str(R_term), ' Ohm, and L_{term} = ', num2str(L_term), ' H' 10 ...
                '' 10 ...
                'Discharging over a load with R_{load} = ', num2str(R_L), ' Ohm and L_{load} = ', num2str(L_L), ' H'],  ... 
                'FontWeight','bold', 'HorizontalAlignment', 'center')
                hold on;
            end
            if N(j) <= 0 || N(j) >= N_max
                figpos = get(gca,'Position');
                text(figpos(1), figpos(2), ...
                ['Not possible to design a PFN Type B Network for given parameters using Rayleigh capacitors of C_i = ', ...
                num2str(C_list(j)), ' Farad.']);
            else
                plot(t, I_L, 'LineWidth', 2)
                hold on;
                plot(t, I_nom, '-', t, I_low, '--', t, I_high, '--', t, 0, '-', t1, (min(I_L)/2: I_high*1.05), '--', t2, (min(I_L)/2: I_high*1.05), '--')
                hold on;
                text(t1, ceil(I_high*1.10), ['t_1 = ', num2str(t1), ' s']);
                text(t2, ceil(I_high*1.10), ['t_2 = ', num2str(t2), ' s']);
                axPos = get(gca,'Position');
                x_start_end = xlim;
                y_start_end = ylim;
                t1_norm = axPos(1) + (t1 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
                t2_norm = axPos(1) + (t2 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
                y1_norm = axPos(2) + (I_nom/2 - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
                y2_norm = axPos(2) + (I_nom/2 - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
                annotation('doublearrow', [t1_norm, t2_norm], [y1_norm, y2_norm]);
                text(t1 + (t2-t1)/4, 0.65*I_nom, ['T_{pulse} = ', num2str(t_pulse), ' s']);
                set(0,'DefaultAxesFontSize', 18)
                legend_text = ['  Rayleigh Network' 10 ...
                '  ----------------' 10 ...
                '  N = ', num2str(N(j)), ' cells' 10 ...
                '  C_{i} = ', num2str(C_list(j)), ' F' 10 ...
                '  L_{i} = ', num2str(L_list(j)), ' H' 10 ...
                '  Z_0 = ', num2str(Z_0(j)), ' Ohm'];
                legend(legend_text, 'location', 'NorthEast')
                xlabel('t (s)')
                ylabel('I_{load} (A)')
            end
        end
        
        %%%%%%%%%% Save Load Current Data %%%%%%%%%%

        if answer{19} == true
            dir = answer{20};
            file = answer{21};
            if strcmp(dir, '') || strcmp(file, '')
                disp('Missing directory or file.')
                return
            end
            if N(j) > 0 && N(j) <= N_max
                path = [dir, '/', file, '_', num2str(N(j)), 'x', num2str(C_list(j)), 'F.txt'];
                ilist = [I_list{j}(:, 1) I_list{j}(:, 2)];
                save(path, 'ilist','-ascii');
            end
        end
    end
end