% vinoptc.m - PFN Type C Optimization
% J. Clementson 2013-04-22

clear

prompt = {'LOAD CHARACTERISTICS'; ...
    ' R_{load} [Ohm]   '; ...
    'L_{load} [Henry]'; ...
    '----------||----------||----------||----------||----------||----------||----------||----------||----------||----------'; ...
    'NETWORK CHARACTERISTICS';...
    'Type of Network'; ...
    'T [s]'; ...
    'C_i [Farad]   '; ...
    'I_{ripple} [%]'; ...
    'L_{imax} [Henry]'; ...
    ' U_0 [Volt]   '; ...
    'R_{term} [Ohm]'; ...
    ' N_{max} [cells]  '; ...
    'L_{term} [Ohm]'; ...
    '----------||----------||----------||----------||----------||----------||----------||----------||----------||----------'; ...
    'DATA OUTPUT';...
    ' T_{int} [s]      '; ...
    ' Plot Load Current'; ...
    ' Save Load Current Data'; ...
    ' Directory   '; ...
    ' File Name [.txt]     '};

name = 'PFN TYPE C OPTIMIZATION';

formats = struct('type', {}, 'style', {}, 'items', {}, ...
  'format', {}, 'limits', {}, 'size', {});

% Load Characteristics
formats(3, 1).type = 'text';

% Load Resistance
formats(5, 1).type   = 'edit';
formats(5, 1).format = 'float';
formats(5, 1).size = 60;

% Load Inductance
formats(5, 2).type = 'edit';
formats(5, 2).format = 'float';
formats(5, 2).size = 60;

% Network Characteristics
formats(7, 1).type = 'text';
formats(7, 2).type = 'none';
formats(7, 2).limits = [0, 10];
formats(7, 3).type = 'none';
formats(7, 3).limits = [0, 10];
formats(8, 1).type = 'text';
formats(8, 2).type = 'none';
formats(8, 2).limits = [0, 10];
formats(8, 3).type = 'none';
formats(8, 3).limits = [0, 10];

% Network Option
formats(10, 1).type = 'list';
formats(10, 1).style = 'radiobutton';
formats(10, 1).items = {' Guillemin                     '};
formats(11, 1).type = 'none';
formats(11, 1).limits = [10, 0];
formats(12, 1).type = 'none';
formats(12, 1).limits = [10, 0];

% Pulse Length
formats(10, 2).type = 'edit';
formats(10, 2).format = 'float';
formats(10, 2).size = 60;

% Cell Capacitors
formats(10, 3).type   = 'list';
formats(10, 3).style  = 'listbox';
formats(10, 3).size = 120;
formats(10, 3).limits = [0, 10];
caplist = importdata('caplist.txt');
formats(10, 3).items  = caplist;
formats(11, 3).type = 'none';
formats(11, 3).limits = [10, 0];
formats(12, 3).type = 'none';
formats(12, 3).limits = [10, 0];
formats(13, 3).type = 'none';
formats(13, 3).limits = [10, 0];
formats(14, 3).type = 'none';
formats(14, 3).limits = [10, 0];

% Pulse Ripple
formats(11, 2).type = 'edit';
formats(11, 2).format = 'float';
formats(11, 2).size = 60;

% Max Cell Inductance
formats(12, 2).type = 'edit';
formats(12, 2).format = 'float';
formats(12, 2).size = 60;

% Voltage Source
formats(13, 1).type = 'edit';
formats(13, 1).format = 'float';
formats(13, 1).size = 60;

% Termination Resistor
formats(13, 2).type = 'edit';
formats(13, 2).format = 'float';
formats(13, 2).size = 60;

% Max Number of Cells
formats(14, 1).type = 'edit';
formats(14, 1).format = 'float';
formats(14, 1).size = 60;

% Termination Inductor
formats(14, 2).type = 'edit';
formats(14, 2).format = 'float';
formats(14, 2).size = 60;

% Data Output
formats(16, 1).type = 'text';
formats(16, 2).type = 'none';
formats(16, 2).limits = [0, 10];
formats(16, 3).type = 'none';
formats(16, 3).limits = [0, 10];
formats(17, 1).type = 'text';
formats(17, 2).type = 'none';
formats(17, 2).limits = [0, 10];
formats(17, 3).type = 'none';
formats(17, 3).limits = [0, 10];

% Time Interval
formats(19, 1).type = 'edit';
formats(19, 1).format = 'float';
formats(19, 1).size = 60;

% Plot options
formats(21, 1).type = 'check';
formats(21, 1).style = 'checkbox';

% Data to file
formats(23, 1).type = 'check';
formats(23, 1).style = 'checkbox';

% Directory
formats(23, 2).type = 'edit';
formats(23, 2).format = 'dir';
formats(23, 2).size = 120;

% File name
formats(23, 3).type = 'edit';
formats(23, 3).format = 'text';
formats(23, 3).size = 120;

defaultanswer = {'', ...        % text      1
    1, ...                      % R_L       2
    0, ...                      % L_L       3
    '', ...                     % frame     4
    '', ...                     % text      5
    1, ...                      % Guillemin 6
    10e-6, ...                  % T         7
    '', ...                     % C_i       8
    10, ...                     % ripple    9
    10e-4, ...                  % L_imax    10
    1000, ...                   % U_0       11
    0, ...                      % R_term    12
    1, ...                      % N_max     13
    0, ...                      & L_term    14
    '', ...                     % frame     15
    '', ...                     % text      16
    50e-6, ...                  % T_int     17
    1, ...                      % plot I_L  18
    0, ...                      % save data 19
    '', ...                     & directory 20
    ''};                        % file      21

options.Resize = 'off';
options.WindowStyle = 'normal';
options.Interpreter = 'tex';
options.CancelButton = 'on';
options.ApplyButton = 'off';
options.AlignControls = 'on';

[answer, canceled] = inputsdlg(prompt, name, formats, defaultanswer, options);

R_L = answer{2};
L_L = answer{3};
T = answer{7};
C_list = caplist(answer{8});
I_rip = answer{9}/100;
U_0 = answer{11};
L_imax = answer{10};
R_term = answer{12};
I_nom = U_0/2/(R_L+R_term);
I_low = I_nom - I_nom*I_rip;
I_high = I_nom + I_nom*I_rip;
N_max = answer{13};
L_term = answer{14};
T_int = [0 answer{17}];
if T_int(2) < T
    disp('Simulation length must be longer than pulse length.')
    return
end

if canceled == true
    return
end


%%%%%%%%%% GUILLEMIN OPTIMIZATION %%%%%%%%%%

if answer{6} == 1
    N = 1;
    L_guillemin = (R_term + R_L)*(T*1.1)/4;
    t_pulse = 0;
    C_guillemin = zeros(1, N_max);
    L_tot = 0;
    C_tot = 0;
    while t_pulse < T && N <= N_max
        L = L_guillemin*ones(1, N);
        if L_guillemin > L_imax
            L = L_imax*ones(1, N);
        end
        C_guillemin(N) = 4*(T*1.1)/(2*N-1)^2/pi^2/(R_term + R_L);
        ind = find((C_list-C_guillemin(N)) > 0, 1, 'first');
        if isempty(ind)
            disp('No suitable capacitors available.')
            return
        end
        C(N) = C_list(ind);
        C_tot = C_tot + C(N);
        R = zeros(1, N);
        u0 = U_0*ones(1, 2*N);
        u0(1, 2:2:2*N) = 0;
        [t, u] = ode45(@pfnc, T_int, u0, [], R_L, L_L, L, C, R, N, R_term, L_term);
        i = zeros(length(t), N);
        for k = 1: N
            i(:, k) = -C(k)*u(:, 2*k);
        end
        I_L = sum(i, 2);
        m = 0;
        t1 = 0;
        t2 = 0;
        for n = 1: length(I_L)
            if I_low <= I_L(n) && I_L(n) <= I_high && t_pulse < T
                m = m + 1;
                if m == 1
                    t1 = t(n);
                else
                    t2 = t(n);
                    t_pulse = t2 - t1;
                end
            else
                m = 0;
            end
        end
        if t_pulse < T
            N = N + 1;
        end
    end

    if N <= N_max
        L_tot = L(1)/N;
        Z_0 = sqrt(L_tot/C_tot);
    end

    %%%%%%%%%% Plot Load Currents %%%%%%%%%%

    if answer{18} == 1
        figure('Color', 1/255*[50 200 250]);
        hold on;
        title(['PFN Type C with U_0 = ', num2str(U_0), ' V, N_{max} = ', num2str(N_max), ' cells, T_{pulse} > ', num2str(T), ' s, I_{ripple} < ', num2str(I_rip*100), ' %, L_{imax} = ', num2str(L_imax), ' H, R_{term} = ', num2str(R_term), ' Ohm, and L_{term} = ', num2str(L_term), ' H' 10 ...
        '' 10 ...
        'Discharging over a load with R_{load} = ', num2str(R_L), ' Ohm and L_{load} = ', num2str(L_L), ' H'],  ... 
        'FontWeight','bold', 'HorizontalAlignment', 'center')
        if N > N_max
            hold on;
            annotation('textbox', get(gca,'Position'), 'String', 'Not possible to design a PFN Type C Network for given parameters.', 'LineStyle', 'none', 'Color', 'r');
        else
            plot(t, I_L, 'LineWidth', 2)
            hold on;
            plot(t, I_nom, '-', t, I_low, '--', t, I_high, '--', t, 0, '-', t1, (min(I_L)/2: I_high*1.05), '--', t2, (min(I_L)/2: I_high*1.05), '--')
            hold on;
            text(t1, ceil(I_high*1.10), ['t_1 = ', num2str(t1), ' s']);
            text(t2, ceil(I_high*1.10), ['t_2 = ', num2str(t2), ' s']);
            axPos = get(gca,'Position');
            x_start_end = xlim;
            y_start_end = ylim;
            t1_norm = axPos(1) + (t1 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
            t2_norm = axPos(1) + (t2 - x_start_end(1))/(x_start_end(2) - x_start_end(1))*axPos(3);
            y1_norm = axPos(2) + (I_nom/2 - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
            y2_norm = axPos(2) + (I_nom/2 - y_start_end(1))/(y_start_end(2) - y_start_end(1))*axPos(4);
            annotation('doublearrow', [t1_norm, t2_norm], [y1_norm, y2_norm]);
            text(t1 + (t2-t1)/4, 0.65*I_nom, ['T_{pulse} = ', num2str(t_pulse), ' s']);
            hold on;
            set(0,'DefaultAxesFontSize', 18)
            legend_text = ['  Guillemin Network' 10 ...
            '  ----------------' 10 ...
            '  N = ', num2str(N), ' cells' 10 ...
            '  C_{ideal} = ', num2str(C_guillemin(1:N)), ' F' 10 ...
            '  C_{i} = ', num2str(C), ' F' 10 ...
            '  L_{i} = ', num2str(L), ' H' 10 ...
            '  Z_0 = ', num2str(Z_0), ' Ohm'];
            legend(legend_text, 'location', 'NorthEast')
            xlabel('t (s)')
            ylabel('I_{load} (A)')
        end
    end
    
    %%%%%%%%%% Save Load Current Data %%%%%%%%%%

    if answer{19} == 1
        dir = answer{20};
        file = answer{21};
        if strcmp(dir, '') || strcmp(file, '')
            disp('Missing directory or file.')
            return
        end
        if N <= N_max
            path = [dir, '/', file, '.txt'];
            tI_L = [t I_L];
            save(path, 'tI_L','-ascii');
        end
    end
end