%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VINPFN – VINETA PULSE FORMING NETWORK ANALYSIS
%
% vinpfn.m, pfna.m, pfnb.m, pfnc.m,
% vingena.m, vingenb.m, vingenc.m,
% vinopta.m, vinoptb.m, vinoptc.m,
% + caplist.txt & manual.pdf
%
% pfnb is based on the fuenfglied function by M. Rainhart
% Modellierung und experimentelle Untersuchung eines kapazitiven Energiespeichers
% BSc Thesis, Ruhr-Universität Bochum (2008)
%
% Data input is handled using the inputsdlg v1.3 function by T. Ikuma
% http://www.mathworks.com/matlabcentral/fileexchange/
% /25862-inputsdlg-enhanced-input-dialog-box-v1-3
%
% If you use these scripts, please reference the following paper,
% which contains a description of the VINPFN application:
%
% Design of A, B, and C Pulse Forming Networks
% J. Clementson, K. Rahbarnia, O. Grulke, and T. Klinger
% submitted to IEEE Transactions on Power Electronics
%
% May 2013
%
% Joel Clementson
% Max Planck Institute for Plasma Physics
% EURATOM Association
% DE-17491 Greifswald, Germany
% joel.clementson@ipp.mpg.de
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear

prompt = {'Type of Network'; ...
    'Type of Calculation'};

name = 'VINETA PULSE FORMING NETWORK ANALYSIS';

formats = struct('type', {}, 'style', {}, 'items', {}, ...
  'format', {}, 'limits', {}, 'size', {});

formats(2, 1).type = 'list';
formats(2, 1).style = 'radiobutton';
formats(2, 1).items = {'  PFN A                ', '  PFN B                ', ...
    '  PFN C                '};

formats(5, 1).type = 'list';
formats(5, 1).style = 'radiobutton';
formats(5, 1).items = {'  General              ', '  Optimization '};

defaultanswer = {1, 1};

options.Resize = 'off';
options.WindowStyle = 'normal';
options.Interpreter = 'tex';
options.CancelButton = 'on';
options.ApplyButton = 'off';
options.AlignControls = 'on';

[answer, canceled] = inputsdlg(prompt, name, formats, defaultanswer, options);

if canceled == true
    return
end

if answer{1} == 1
    if answer{2} == 1
        vingena
    else
        vinopta
    end
end

if answer{1} == 2
    if answer{2} == 1
        vingenb
    else
        vinoptb
    end
end

if answer{1} == 3
    if answer{2} == 1
        vingenc
    else
        vinoptc
    end
end

vinpfn